from websucker.agent import Connection,visit_links,visit_domain
from websucker.agent import ParsedDocument
from websucker.parser import BaseParser
from websucker.parser import normalize_link,urlunparse
from websucker.db import Data
from websucker.db import get_schema
import click
import pprint



def create_database_from_context(ctx):
    return Data(ctx.obj["cassandra_keyspace"],ctx.obj["cassandra_host"],ctx.obj["cassandra_port"])

@click.group()
@click.pass_context
@click.option("--cassandra-keyspace",metavar="CASSANDRA_KEYSPACE",help="cassandra keyspace (if defined, value read from CASSANDRA_KEYSPACE env variable)",envvar="CASSANDRA_KEYSPACE",default="websucker",show_default=True)
@click.option("--cassandra-host",metavar="CASSANDRA_HOST",help="cassandra host (if defined, value read from CASSANDRA_HOST env variable)",envvar="CASSANDRA_HOST",default="127.0.0.1",show_default=True)
@click.option("--cassandra-port",metavar="CASSANDRA_PORT",help="cassandra port (if defined, value read from CASSANDRA_PORT env variable)",envvar="CASSANDRA_PORT",default=9042,show_default=True)

@click.option("--justext-language",metavar="JUSTEXT_LANGUAGE",help="Target language (if defined, value read from JUSTEXT_LANGUAGE env variable)",envvar="JUSTEXT_LANGUAGE",default="English",show_default=True)
@click.option("--visit",is_flag=True)
def cli(ctx,cassandra_keyspace,cassandra_host,cassandra_port,justext_language,visit):
    ctx.ensure_object(dict)
    p = BaseParser()
    p.justext_language = justext_language
    ctx.obj["parser"] = p
    ctx.obj["cassandra_host"] = cassandra_host
    ctx.obj["cassandra_port"] = cassandra_port
    ctx.obj["cassandra_keyspace"] = cassandra_keyspace
    ctx.obj["visit"] = visit


@cli.command(help="Print domains")
@click.pass_context
@click.argument("count",type=int,default=20)
def all(ctx,count):
    p = ctx.obj["parser"]
    c = Connection()
    db = create_database_from_context(ctx)
    res = db.all_domains(count)
    for row in res:
        print(",".join(map(str,row)))
        if ctx.obj["visit"]:
            visit_domain(row[0],p,db)

@cli.command(help="Continue crawling of seen links from a domain")
@click.pass_context
@click.argument("domain")
def crawl(ctx, domain):
    db = create_database_from_context(ctx)
    p = ctx.obj["parser"]
    c = Connection()
    links = db.get_visit_links(domain,p.recent_links,p.old_links,p.random_links)
    visit_links(links,c,p,db)
    db.check_domain(domain)

@cli.command(help="find best domains")
@click.pass_context
@click.argument("count",type=int,default=20)
#@click.option("visit",is_flag=True)
def best(ctx, count):
    db = create_database_from_context(ctx)
    p = ctx.obj["parser"]
    domains = db.get_best_domains(count)
    for domain,gr in domains:
        print(domain,gr)
        if ctx.obj["visit"]:
           visit_domain(domain,p,db)


@cli.command(help="Find unvisited domains, Visit a site, get links and crawl")
@click.pass_context
@click.argument("count",type=int,default=20)
def unvisited(ctx, count):
    db = create_database_from_context(ctx)
    p = ctx.obj["parser"]
    c = Connection()
    domains = db.get_unvisited_domains(count)
    for domain in domains:
        print(domain)
        if ctx.obj["visit"]:
          visit_domain(domain,p,db)

@cli.command(help="Visit url, get links and crawl. Start here")
@click.pass_context
@click.argument("link")
def visit(ctx, link):
    db = create_database_from_context(ctx)
    p = ctx.obj["parser"]
    c = Connection()
    nl = normalize_link(link)
    domain=nl[1]
    visit_domain(domain,p,db)

@cli.command(help="Update domain statistics")
@click.pass_context
@click.argument("domain")
def check(ctx,domain):
    db = create_database_from_context(ctx)
    res = db.check_domain(domain)
    print(res)

@cli.command(help="Print daily report")
@click.pass_context
def report(ctx):
    db = create_database_from_context(ctx)
    db.daily_report()

@cli.command(help="Print keyspace schema")
def schema():
    schema = get_schema()
    print(schema)

@cli.command(help="Fetch given url (just for debug)")
@click.pass_context
@click.argument("urls")
def fetch(ctx,urls):
    parser = ctx.obj["parser"]
    # Visit first page
    connection = Connection()
    responses = connection.html_download2(urls)
    for res in responses:
        target_link = res.get_canonical()
        pd = ParsedDocument(parser,target_link)
        pd.extract(res.content, res.bs)
        print(pd)

if __name__ == "__main__":
    cli()
